/* assemble_buffer() -- takes fragments of messages (i.e. packets) in
 * the form of a linked list and assembles them into a complete message
 * (i.e. a buffer).  The function returns pointer the next packet containing
 * a new message.  The calling program needs to ensure that the *p_list
 * points to the *first* packet of a message.
 */
#define _XOPEN_SOURCE 

#include <stdlib.h>
#include <string.h>

#include "cd4port.h"
#include "cad4io.h"



CommList *assemble_buffer( CommList *p_list, CAD4_Buffer *mesg )
{

    char          tx_direction;  /* transmission direction, '>' or '<'  */
    int           first_tx = 1;
    size_t        total_mesg_bytes = 0,
                  io_bytes;
    CommList      *current;
    unsigned char *cur_mesg_pos,
                  *cur_pkt_pos;
  
    

    current = p_list;
    tx_direction = current->pkt.direction;  /* tag the direction of this
                                             * data transfer
                                             */
    
    cur_mesg_pos = mesg->data.c;
    cur_pkt_pos = current->pkt.buffer;

    while( tx_direction == current->pkt.direction ) {
       io_bytes = current->pkt.io_bytes;
       if( first_tx ) {
           mesg->header_byte = current->pkt.buffer[HEADER_BYTE];
           memcpy( &(mesg->length.data_len), &(current->pkt.buffer[LOW_BYTE]),
                   sizeof(unsigned short) );
           cur_pkt_pos = &(current->pkt.buffer[START_DATA]); 
           io_bytes -= (sizeof(unsigned short) + 1);
           total_mesg_bytes += (sizeof(unsigned short) + 1);
           first_tx = 0;  /* don't do this again */
       }
       memcpy( cur_mesg_pos, cur_pkt_pos, io_bytes );
       cur_mesg_pos += io_bytes;
       total_mesg_bytes += io_bytes;
       current = current->next;
    }

    return current;
}
