#include <sys/types.h>
#include <stdio.h>

/* include cad4io.h header file */
#include "cad4io.h"

/* some useful #define's */
#define LOG_FILE_NAME  "cd4port.log"

/* some defines for indication the direction of the 
 * commumication.
 */
#define SERIAL_2_HOST '>'
#define HOST_2_SERIAL '<'
#define INIT_DIRECTION '*'

#define FILE_NAME_LEN  24
#define READ_BUFFER_LEN  8

#define CD4PORT_PID    "/var/cd4port."   /* where cd4logger picks up cd4port's
                                          * PID for sending the HUP signal
                                          */

#define CD4LOGGER_PID  "/var/cd4logger." /* where cd4port picks up cd4logger's
                                          * PID for sending signals.
                                          */
#define BUFFER_SIZE   (IO_DATA_LEN + IO_HEADER_LEN )
#define DEVICE_PREFIX "/dev/"
#define DEFAULT_SPEED  9600
#define DEFAULT_PARITY -1

#define CONFIG_DIR     "/dataQ/config/"
#define CD4_NR_TOKEN   'Q'
#define RS232_FILE     "rs232_port"
#define CHAR_FORMAT     0
#define HEX_FORMAT      1
#define UINT_FORMAT     2
#define SHORT_FORMAT    3


/* this is the packet struct which contains the data buffer as
 * well as information containing pertinent to the cd4port
 * program.
 */
struct comm_packet {
       unsigned int     n_seq;
       size_t           io_bytes;  /* total number of bytes read from port */
       char             direction;
       unsigned char    buffer[IO_DATA_LEN + IO_HEADER_LEN];
       };

typedef struct comm_packet COMM_Packet;

/* a couple of linked lists for the raw packets and
 * assembled buffers.
 */
struct comm_list {
       struct comm_list *next;
       COMM_Packet pkt;
       };
typedef struct comm_list CommList;


struct buffer_list {
       struct buffer_list *next;
       CAD4_Buffer buf;
       };
typedef struct buffer_list BufferList;


/* prototypes */
/* terminals.c */
extern char *get_cad4_terminal( char *cad4_nr, char *home_dir );
extern int terminal_init( int fd, int speed, int parity );
extern int terminal_cleanup( int fd );

/* serial_comm.c */
extern int serial_comm(int rd_fd, int wr_fd, COMM_Packet *packet);

/* print_packet.c */
extern void init_packet( COMM_Packet *p );
extern void print_cad4_packet(FILE *out, CommList *pack, int format );
extern CommList *assemble_cad4_buffer( CommList *p_list, CAD4_Buffer *mesg,
                                   size_t *n_bytes );

/* keyboard.c */
extern int non_canonical_kybd(void);
extern void restore_kybd(void);
extern char key_hit(void);
/* misc.c */
extern void sigusr_handler(int s);
extern void sigusr2_handler(int s);
extern void sighup_handler(int s);
extern char *create_pid_file_name(char *path, char *cad4_nr);
extern void destroy_pid_file(char *path);
extern void write_pid_file(char *pid_file);
extern pid_t get_pid_file(char *file);
extern void fill_cad4_data_block( CAD4_Buffer *buf,
                                  struct cad4_data_block *blk );
extern void print_cad4_data_block( FILE *out, struct cad4_data_block *blk );
extern double conv_hw_words( hw_word w1, hw_word w0 );
