/* these functions control the behavior of the keyboard. */

#define _XOPEN_SOURCE
#include <sys/types.h>
#include <sys/time.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

/* static file scope variables. */
static struct termios initial;
static struct termios current;
static int fd = STDIN_FILENO;

/* functions */
int non_canonical_kybd( void )
{
    int ret_val;
    ret_val = tcgetattr( fd, &initial );
    if( -1 == ret_val ) {
        perror( "tcgetattr()" );
        return ret_val;
    }

    current = initial;
    current.c_lflag &= ~ICANON;
    current.c_lflag &= ~ISIG;
    current.c_cc[VMIN] = 0;
    current.c_cc[VTIME] = 0;

    ret_val = tcsetattr( fd, TCSANOW, &current );
    if( -1 == ret_val ) {
         perror( "tcsetattr()" );
         return ret_val;
    }

    tcflush( fd, TCIFLUSH );
    return 0;
}

/* restores keyboard to it's starting state. Can be used as a signal 
 * handler in case the progam terminates unexpectedly.
 */
void restore_kybd( void )
{

    int ret_val;

    current = initial;

    ret_val = tcsetattr( fd, TCSANOW, &current );
    if( -1 == ret_val ) {
         perror( "tcsetattr()" );
         exit( EXIT_FAILURE );
    }

    tcflush( fd, TCIFLUSH );
    return;

}

/* key_hit() detects that a key has been pressed and returns the character
 * which has been pressed.  If no key has been pressed, '\0' is
 * returned.  It is assumed that the keyboard has been put into non-
 * canonical mode before calling.
 */

char  key_hit( void )
{
    int sel_ret_val;
    int s_set = fd + 1;
    ssize_t n_read;

    char ch;
   
    fd_set  keyboard;
    struct timeval timeout;

    timeout.tv_sec = 0;  /* return immediately */
    timeout.tv_usec = 0;

    FD_ZERO( &keyboard );
    FD_SET( fd, &keyboard );

    

    if( (sel_ret_val = select(s_set, &keyboard, NULL, NULL, &timeout)) < 0 )
         perror( "select()" );

    if( FD_ISSET(fd, &keyboard) ) {
        n_read = read( fd, &ch, 1 );

        if( n_read < 0 ) 
            perror( "read()" );
       
        return ch;    
   
    }
    return '\0';
}
