/* this file contains various functions for getting other processes'
 * PID, and the signal handling functions.
 */

#define _XOPEN_SOURCE
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cd4port.h"

/* sigusr_handler() is installed in the cd4logger process to break out
 * of the pause() function read the shared memory.
 */
void sigusr_handler( int s )
{
    return;
}

/* create_pid_file_name() -- builds the name for the file containing
 * the process' PID.
 */
char *create_pid_file_name( char *path, char *cad4_nr )
{
     size_t len_path,
            len_pw_dir;
     char *file_name;
     struct passwd *info;

     info = getpwuid( getuid() );
     if( !info )
         return NULL;

     len_path = strlen( path );
     len_pw_dir = strlen( info->pw_dir );  
     file_name = malloc( len_path + len_pw_dir + 2 );
     if( !file_name )
         return NULL;

     sprintf( file_name, "%s%s%s", info->pw_dir, path, cad4_nr );
     return file_name;
}

/* destroy_pid_file() -- processes should destroy their own PID file
 * before they exit.
 */
void destroy_pid_file( char *path )
{
    int ret_val;
    ret_val = unlink( path );
    if( -1 == ret_val ) {
        perror( path );
        exit( EXIT_FAILURE );
    }
    free( path );
}
     

/* write_pid_file() is used by the programs to write their own PID's
 * to a file so the other process can find it.
 */
void write_pid_file( char *pid_file )
{
     FILE *pf;

     pf = fopen( pid_file, "w" );
     if( !pf ) {
         perror( pid_file );
         exit( EXIT_FAILURE );
     }
     fprintf( pf, "%d", (int)getpid() );
     fclose( pf );
     return;
}

/* get_pid_file() -- reads a PID from a file and returns it to the 
 * calling function.  Used by cd4logger process.   If there is a failure,
 * function returns a -1.
 */ 
pid_t get_pid_file( char *file )
{
    FILE   *pf;
    char   read_buffer[READ_BUFFER_LEN],
           *ptr;

    memset( read_buffer, '\0', sizeof(read_buffer) );

    pf = fopen( file, "r" );
    if( !pf )
        return -1;

    if( NULL == fgets( read_buffer, sizeof(read_buffer), pf) ) 
        return -1;

    if( NULL != (ptr = strchr(read_buffer, '\n')) )
         *ptr = '\0';
    
    return (pid_t)strtol( read_buffer, (char **)NULL, 10 );
}
