/* function to get the name of terminal port specified in the cad4
 * config subdirectory.  Assume standard inefficient Nonius setup practice
 * of using file names to specify what could be environment variables, 
 * or read from a single config file.
 */
#define _POSIX_SOURCE

#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

/* local headers */
#include "cd4port.h"

char *get_cad4_terminal( char *cad4_nr, char *home_dir )
{
    DIR            *config_d;
    struct dirent  *cd4_conf;
    char           *ptr,
                   *config_dir,
                   *pseudo,
                   *pseudo_device;
    size_t         len;

    config_dir = malloc( strlen(home_dir) + strlen(CONFIG_DIR) + 1 );
    if( !config_dir ) {
      fprintf( stderr, "Memory Allocation Error. %s:%d Exiting.\n", 
               __FILE__, __LINE__ );
      exit( EXIT_FAILURE );
    }
    sprintf( config_dir, "%s%s", home_dir, CONFIG_DIR );
    if( NULL != (ptr = strchr(config_dir, CD4_NR_TOKEN)) )
             *ptr = cad4_nr[0];

/* open the directory where the rs232_port.xxxx file lives */
    config_d = opendir( config_dir );
    if( !config_d ) {
        perror( config_dir );
        exit( EXIT_FAILURE );
    }

/* search the directory for the rs232_port substring in the file name */
   while( NULL != (cd4_conf = readdir( config_d )) ) {
        if( NULL != strstr(cd4_conf->d_name, RS232_FILE) ) {
           pseudo = malloc( len = strlen(cd4_conf->d_name) + 1 );
           if( !pseudo ) {
              fprintf( stderr, " Memory Allocation Error. %s:%d Exiting.\n", 
                       __FILE__, __LINE__ );
              exit( EXIT_FAILURE );
           }
           strcpy( pseudo, cd4_conf->d_name );
           pseudo[len-1] = '\0'; /* ensure terminated string */
           break;
        }
   }

/* find the device name in the file name and preappend the "/dev/" 
 * prefix to it.
 */

   ptr = strchr( pseudo, '.' );
   ptr++;  /* move past the '.' */
   pseudo_device = malloc( strlen(ptr) +  strlen(DEVICE_PREFIX) + 1);
   if( !pseudo_device ) {
        fprintf( stderr, " Memory Allocation Error. %s:%d Exiting.\n", 
                 __FILE__, __LINE__ );
        exit( EXIT_FAILURE );
   }
   sprintf( pseudo_device, "%s%s", DEVICE_PREFIX, ptr );

/* cleanup and return the value */   
   closedir( config_d );
   free( config_dir );
   free( pseudo );

   return pseudo_device;
}
