#define _XOPEN_SOURCE
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <stdio.h>

/* functions for dealing with SYSV semaphores.  All functions return
 * the return value of the semaphore operation.  Caller deals with
 * error.
 */

/* sem_set() -- initializes value for semaphore.
 */

int sem_set( int sem_id )
{
    int ret_val;
    union semun sem_union;

    sem_union.val = 1;
    ret_val = semctl( sem_id, 0, SETVAL, sem_union );
    if( -1 == ret_val ) 
        perror( "semctl()" );
   
    return ret_val; 
}

/* use sem_attempt_lock() to wait and enter a critical section of code */
int sem_attempt_lock( int sem_id )
{
    int ret_val;
    struct sembuf  sb;

    sb.sem_num = 0;
    sb.sem_op = -1;
    sb.sem_flg = SEM_UNDO;

    ret_val = semop( sem_id, &sb, 1 );
    if( -1 == ret_val ) 
        perror( "semop()" );

    return ret_val;
}

/* use sem_release() when finished with critical section of code */
int sem_release( int sem_id )
{
    int ret_val;
    struct sembuf  sb;

    sb.sem_num = 0;
    sb.sem_op = 1;
    sb.sem_flg = SEM_UNDO;

    ret_val = semop( sem_id, &sb, 1 );
    if( -1 == ret_val ) 
        perror( "semop()" );

    return ret_val;
}

/* use sem_delete() to remove the semaphore from the system. */

int sem_delete( int sem_id )
{
    int ret_val;
    union semun sem_union;

    ret_val = semctl( sem_id, 0, IPC_RMID, sem_union );
    if( -1 == ret_val )
         perror( "semctl()" );

     return ret_val;
}
