#define _XOPEN_SOURCE

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "cd4port.h"

/* global variables */
extern  pid_t pid;
extern  char *pid_file_name;
extern logger_attached;

/* sighup_handler() is called by cd4port when a SIGHUP is received indicating
 * that a cd4logger process has attached shared memory and that cd4port
 * should get the cd4logger PID from a file.  We avoid using stdio functions
 * in a signal handler because that's the Right Thing to do.
 */

void sighup_handler( int s )
{
    int fd;
    ssize_t bytes;
    char    read_buffer[READ_BUFFER_LEN];

    fd = open( pid_file_name, O_RDONLY );
    if( -1 == fd ) {
        pid = -1;
        return;
    }

    bytes = read(fd, read_buffer, READ_BUFFER_LEN);
    if( (0 == bytes) || (-1 == bytes) ) {
        close( fd );
        pid = -1;
        return;
    }

    read_buffer[bytes] = '\0';   
    pid = (pid_t)strtol( read_buffer, (char **)NULL, 10 ); 

    close( fd );
}

/* SIGUSR2 acts as a toggle to tell cd4port that a cd4logger job is attached
 * to the shared memory and it should be notified when data is ready to be
 * read from the shared memory segment.
 */
void sigusr2_handler( int s )
{
    if( logger_attached )
        logger_attached = 0;  /* logger has detached */
    else
        logger_attached = 1;  /* logger is attached */

    return;
}
