#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define BUFFER_LEN  12 

struct packet {
       size_t n_bytes;
       char  buffer[BUFFER_LEN];
       };


void *build_packet( char *in, struct packet *data )
{
    static char assemble[BUFFER_LEN];
    static char *ptr_current;
    static int  total_bytes;
    static size_t bytes_remaining;

    int len = 0;

/* initial time through for a given packet, make sure everything is
 * squeaky clean.
 */
    if( 0 == total_bytes ) { 
        memset( assemble, '\0', sizeof(assemble) );
        bytes_remaining = sizeof(assemble);
        ptr_current = assemble;
    }

    len = strlen( in );

/* leave one byte left over for terminating NUL character. */
    if( (bytes_remaining - 1) > len ) {
        memcpy( ptr_current, in, len );
        ptr_current += len;
        total_bytes += len;
        bytes_remaining -= len;
    }
    else {
        memcpy( ptr_current, in, bytes_remaining - 1 );
        total_bytes += bytes_remaining - 1;
        bytes_remaining = 1;
    }

/* if assembly buffer is full then copy data to packet buffer, otherwise
 * return NULL to tell caller that packet is not ready to go.
 */
    if( 1 == bytes_remaining ) { /* give room for '\0' */
        data->n_bytes = total_bytes;
        memcpy( data->buffer, assemble, BUFFER_LEN );
        data->buffer[BUFFER_LEN - 1] = '\0';
        total_bytes = 0;  /* ensure reinitialization for next packet */
    }
    else 
       return NULL;

    return data;  /* return some non-NULL value */
}

void get_input( char *a, int length )
{
    char *ptr;
    fputs( "Enter data: ", stdout );
    fflush( stdout );

    fgets( a, length, stdin );
    if( NULL != (ptr = strchr(a, '\n')) ) *ptr = '\0';
    
    return;
}    

int main( void )
{
    char line[BUFFER_LEN];

    struct packet my_data,
                  *data_ptr = NULL;

    my_data.n_bytes = 0;
    memset( my_data.buffer, '\0', BUFFER_LEN );

    while( NULL == data_ptr ) {
      get_input( line, sizeof(line) );
      data_ptr = build_packet( line, &my_data );
    }

    printf( "Packet has %d bytes.\nData: %s\n", my_data.n_bytes,
                                                my_data.buffer );

    exit( EXIT_SUCCESS );
}
