/* the functions in this file modify the behavior of stdin.
 * Generally note:  if a system call fails, functions return -1, 
 * and caller must deal with the error.  Successful completion
 * returns 0.
 */ 
#define _POSIX_SOURCE

#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>

static int stdin_fd;

static struct termios initial, 
                      current;

int set_stdin_noncanon_mode( void )
{
    stdin_fd = fileno( stdin );
    if( !isatty(stdin_fd) )  return -1;

    if( -1 == tcgetattr(stdin_fd, &initial) ) {
       fprintf( stderr, "%s:%d -- tcgetattr(): %s\n", 
                __FILE__, __LINE__, strerror(errno) );
       return -1;
    }
    current = initial;
    current.c_lflag &= ~(ECHO | ICANON);
    current.c_cc[VMIN] = 1;
    current.c_cc[VTIME] = 0;

    if( -1 == tcsetattr(stdin_fd, TCSANOW, &current) ) {
       fprintf( stderr, "%s:%d -- tcsetattr(): %s\n", 
                __FILE__, __LINE__, strerror(errno) );
       return -1;
    }
    
    return 0; 
}

int restore_stdin( void )
{
    if( -1 == tcsetattr(stdin_fd, TCSANOW, &initial) ) {
       fprintf( stderr, "%s:%d -- tcsetattr(): %s\n", 
                __FILE__, __LINE__, strerror(errno) );
       return -1;
    }

    return 0;
}

/* use this function to restore stdin in the case of unexpected 
 * program termination (i.e. a signal handler function). */
 */
void cleanup_stdin( int sig )
{
    tcsetattr( stdin_fd, TCSANOW, &initial );
    return;
}
