/* this code takes care of setting the serial port and pseudoterminals
 */

#define _POSIX_SOURCE

#include <termios.h>
#include <unistd.h>
#include <stdio.h>

#include "cd4port.h"

int terminal_init( int fd, int speed, int parity )
{

   struct termios term_set;

/* Get the initial port settings */
    tcgetattr( fd, &term_set );


/* Set the serial port attributes.  Use the terminal settings from a
 * working Nonius CAD4-linux installation (at NCSU).  Then modify using 
 * parameters input into this function by calling routine.  
 * I tried to modify only POSIX.1 specified flags, and leave other
 * flags as they are.  However, this is not possible.  There are 2 non-POSIX
 * flags which need to be disabled (ECHOCTL and ECHOKE).
 */
    term_set.c_iflag &= ~( IGNBRK |
                          BRKINT |
                          IGNPAR |
                          PARMRK |
                          INPCK  |
                          ISTRIP |
                          INLCR  |
                          IGNCR  |
                          ICRNL  |
                          IXON   |
                          IXOFF );

    term_set.c_oflag &= ~(OPOST); 

    term_set.c_cflag &= ~( PARENB |
                          PARODD |
                          CSTOPB );

    term_set.c_cflag |= CLOCAL | CREAD | CS8 | HUPCL;

    term_set.c_lflag &= ~( ISIG   |
                          ICANON |
                          IEXTEN |
                          ECHO   |
                          ECHOE  |
                          ECHOK  |
                          ECHONL |
                          NOFLSH |
                          ECHOCTL |    /* non posix flag */
                          ECHOKE |     /* non posix flag */
                          TOSTOP );

/* Set the time and minimum number of characters before wake up */

    term_set.c_cc[VMIN] = 0;
    term_set.c_cc[VTIME] = 10;  /* This may have to tweaked some */

/* Now modify according to command line arguments */

      switch( parity ) {
        case 'o':   /* FALL THROUGH */
        case 'O':     
           term_set.c_cflag |= PARENB | PARODD;
           break;
        case 'e':   /* FALL THROUGH */
        case 'E':
           term_set.c_cflag |= PARENB;
           term_set.c_cflag &= ~( PARODD );
           break;
        case -1:    /* FALL THROUGH, and do nothing. */
       default:    /* do nothing */
            break;  
    }

/* Add support for all terminal speeds in termios.h (POSIX) */

    switch( speed ) {
        case 19200:
           cfsetispeed( &term_set, B19200 );
           cfsetospeed( &term_set, B19200 );
           break;
           default:  /* FALL THROUGH: 9600 is the DEFAULT baud setting */
        case  9600:
           cfsetispeed( &term_set, B9600 );
           cfsetospeed( &term_set, B9600 );
           break;
        case  4800:
           cfsetispeed( &term_set, B4800 );
           cfsetospeed( &term_set, B4800 );
           break;
        case  2400:
           cfsetispeed( &term_set, B2400 );
           cfsetospeed( &term_set, B2400 );
           break;
        case  1200:
           cfsetispeed( &term_set, B1200 );
           cfsetospeed( &term_set, B1200 );
           break;
        case   600:
           cfsetispeed( &term_set, B600 );
           cfsetospeed( &term_set, B600 );
           break;
        case   300:
           cfsetispeed( &term_set, B300 );
           cfsetospeed( &term_set, B300 );
           break;
        case   200:
           cfsetispeed( &term_set, B200 );
           cfsetospeed( &term_set, B200 );
           break;
        case   150:
           cfsetispeed( &term_set, B150 );
           cfsetospeed( &term_set, B150 );
           break;
        case   134:
           cfsetispeed( &term_set, B134 );
           cfsetospeed( &term_set, B134 );
           break;
        case   110:
           cfsetispeed( &term_set, B110 );
           cfsetospeed( &term_set, B110 );
           break;
        case    75:
           cfsetispeed( &term_set, B75 );
           cfsetospeed( &term_set, B75 );
           break;
        case    50:
           cfsetispeed( &term_set, B50 );
           cfsetospeed( &term_set, B50 );
           break;
        case     0:
           cfsetispeed( &term_set, B0 );
           cfsetospeed( &term_set, B0 );
           break;
    }

/* Now actually set the parameters */

    if( -1 == tcsetattr( fd, TCSANOW, &term_set )) {
         perror("Error setting serial port");
         return -1;  /* error, caller deals with it */
    }

    tcflush( fd, TCIOFLUSH );
    return 0;  /* ok, let caller deal with I/O */
}


int terminal_cleanup( int fd )
{
    tcflush( fd, TCIOFLUSH );
    return close( fd );  /* let caller deal with any error */
}

