/* implement the bit packing scheme suggested by Sean A. */

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

/* these defines are from the DIFRAC code. */
#define  DGRST (360.0 /(128.0 * 4096.0))  /* encoder -> degrees */
#define  STDGR (128.0 * 4096.0 / 360.0)   /* degrees -> encoder */

unsigned short get_short( unsigned long x, int p, int n );

int main( void )
{
    union ifl {
          float fval;
          long  ulval;
          };

     union ifl data;
     long lower_bits = 0;
     short s[2];
     float angle;

     s[0] = 0xaaab;
     s[1] = 0x0000;

     data.ulval = 0;

     data.ulval |= (long)s[1] << (CHAR_BIT * sizeof(short));
     lower_bits = (long)s[0];
     data.ulval |= lower_bits;
 
     angle = data.fval;
 
     fprintf( stdout, "Unsigned long: %#lx\n", data.ulval );
     fprintf( stdout, "Float:  %8.3f\n", angle / STDGR );

/* now do the back conversion following the getbits() function as 
 * seen on page 49 of K&R2.
 */
     s[0] = 0;
     s[1] = 0;

     data.fval = 78.0 * STDGR;

     s[0] = get_short( data.ulval, 15, 16 );
     s[1] = get_short( data.ulval, 31, 16 );

     fprintf( stdout, "Bitshifted value: %#hx\n", s[0] );
     fprintf( stdout, "Bitshifted value: %#hx\n", s[1] );

     exit( EXIT_FAILURE );
}  

unsigned short get_short( unsigned long x, int p, int n )
{
    return ( x >> (p+1-n) & ~(~0 << n) );
}
